/* Routines for constructing cubic splines
 * This file is part of David Deaven's personal library.
 * Copyright (c) 1994 David Deaven
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Id: readspline.c,v 1.3 1995/10/10 15:03:39 deaven Exp $
 */

#include <stdio.h>
#include "dmd/memory.h"
#include "xyplot.h"

#define CHUNK 256

/* Read a spline from the data in xyplot(1) file fn, columns xc,yc.
 */
Spline1D *ReadSplineXY(char *fn, int xc, int yc)
{
	int i, n, nd;
	double *xd, *yd, x, y, xerr, yerr;
	XYFileInfo fi;
	Spline1D *s;

	fi.name = fn;
	fi.xcol = fi.xcolmax = xc;
	fi.ycol = fi.ycolmax = yc;
	fi.xerr = fi.yerr = 0;

	if(XYStartStream(&fi)) return NULL;
	n = nd = 0;
	xd = yd = NULL;
	while(XYReadStream(&fi, &x, &y, &xerr, &yerr) != EOF)
	{
	  if(++n > nd) {
	    nd += CHUNK;
	    xd = myrealloc(xd, nd * sizeof(double));
	    yd = myrealloc(yd, nd * sizeof(double));
	  }
	  xd[n - 1] = x;
	  yd[n - 1] = y;
	}

	s = newSpline1D(n);
	for(i=0; i<n; ++i)
	{
	  s->x[i] = xd[i];
	  s->y[i] = yd[i];
	}
	myfree(xd);
	myfree(yd);
	return s;
}
